#!/usr/bin/env node

// src/bin/index.ts
import { parseArgs } from "util";

// src/bin/commands/init.ts
import childProcess from "child_process";
import fs from "fs/promises";
import json5 from "json5";
import ora from "ora";
import path from "path";
import util from "util";
var exec = util.promisify(childProcess.exec);
var exists = async (file) => {
  try {
    await fs.access(path.join(process.cwd(), file));
  } catch (err) {
    if (err.code === "ENOENT") {
      return false;
    } else {
      throw err;
    }
  }
  return true;
};
var init_default = async (positionals2) => {
  const spinner = ora("Initializing project").start();
  const lastPositional = positionals2[positionals2.length - 1];
  const spaceHandle = lastPositional === "init" ? null : lastPositional;
  if (!spaceHandle) {
    spinner.fail("Please provide a space handle like this:\n$ ronin init my-space");
    process.exit(1);
  }
  if (!await exists("package.json")) {
    spinner.fail("No `package.json` found in the current directory. Please run the command in your project.");
    process.exit(1);
  }
  const packageManager = await exists("bun.lockb") ? "bun" : "npm";
  const packageManagerName = packageManager === "bun" ? "Bun" : "npm";
  spinner.text = `Detected ${packageManagerName} \u2014 installing types package`;
  try {
    if (packageManager === "bun") {
      await exec(`bun add @ronin/${spaceHandle} --dev`);
    } else {
      await exec(`npm install @ronin/${spaceHandle} --save-dev`);
    }
    const tsConfigPath = path.join(process.cwd(), "tsconfig.json");
    const tsConfigExists = await exists("tsconfig.json");
    if (!tsConfigExists) return;
    const contents = await fs.readFile(tsConfigPath, "utf-8");
    const tsConfig = json5.parse(contents);
    if (!tsConfig.compilerOptions.types || !tsConfig.compilerOptions.types.includes(`@ronin/${spaceHandle}`))
      Object.assign(tsConfig.compilerOptions, {
        types: [...tsConfig.compilerOptions.types || [], `@ronin/${spaceHandle}`]
      });
    await fs.writeFile(tsConfigPath, JSON.stringify(tsConfig, null, 2));
  } catch (err) {
    if (err instanceof Error && err.message.includes("401")) {
      spinner.fail(`You are not a member of the "${spaceHandle}" space or the space doesn't exist.`);
      process.exit(1);
    }
  }
  spinner.succeed("Project initialized");
};

// src/bin/commands/login.ts
import getPort from "get-port";
import http from "http";
import open from "open";
import ora2 from "ora";

// src/bin/utils/session.ts
import toml from "@iarna/toml";
import fs2 from "fs/promises";
import ini from "ini";
import os from "os";
import path2 from "path";
var CACHE_DIR = path2.join(os.homedir(), ".ronin");
var CACHE_DIR_FILE = path2.join(CACHE_DIR, "session.json");
var getSession = async () => {
  try {
    const contents = await fs2.readFile(CACHE_DIR_FILE, {
      encoding: "utf-8"
    });
    const parsedContents = JSON.parse(contents);
    if (parsedContents?.token) return parsedContents;
  } catch (err) {
    if (err.code !== "ENOENT") throw err;
  }
  return null;
};
var readConfigFile = async (filePath, name, parser) => {
  let npmConfigContents = {};
  try {
    const contents = await fs2.readFile(filePath, { encoding: "utf-8" });
    npmConfigContents = parser(contents);
  } catch (err) {
    if (err.code !== "ENOENT") {
      console.error(`Failed to read ${name} config file at ${filePath}`, err);
    }
  }
  return npmConfigContents;
};
var writeConfigFile = async (filePath, contents) => {
  return fs2.writeFile(filePath, contents, { encoding: "utf-8" });
};
var storeSession = async (token) => {
  try {
    await fs2.stat(CACHE_DIR);
  } catch (err) {
    if (err.code === "ENOENT") {
      await fs2.mkdir(CACHE_DIR);
    } else {
      throw err;
    }
  }
  return writeConfigFile(CACHE_DIR_FILE, JSON.stringify({ token }, null, 2));
};
var storeTokenForNPM = async (token) => {
  const npmConfigFile = process.env["npm_config_userconfig"] || path2.join(os.homedir(), ".npmrc");
  const npmConfigContents = await readConfigFile(npmConfigFile, "npm", ini.parse);
  npmConfigContents["@ronin:registry"] = "https://ronin.supply";
  npmConfigContents["//ronin.supply/:_authToken"] = token;
  await writeConfigFile(npmConfigFile, ini.stringify(npmConfigContents));
};
var storeTokenForBun = async (token) => {
  const bunConfigFile = path2.join(os.homedir(), ".bunfig.toml");
  const bunConfigContents = await readConfigFile(bunConfigFile, "Bun", toml.parse);
  if (!bunConfigContents.install) bunConfigContents.install = {};
  if (!bunConfigContents.install.scopes) bunConfigContents.install.scopes = {};
  if (!bunConfigContents.install.scopes.ronin) bunConfigContents.install.scopes.ronin = {};
  bunConfigContents.install.scopes.ronin.url = "https://ronin.supply";
  bunConfigContents.install.scopes.ronin.token = token;
  await writeConfigFile(bunConfigFile, toml.stringify(bunConfigContents));
};

// src/bin/commands/login.ts
var login_default = async (appToken) => {
  const spinner = ora2("Logging in").start();
  if (appToken) {
    await Promise.all([storeTokenForNPM(appToken), storeTokenForBun(appToken)]);
    spinner.succeed("Logged in successfully!");
    return;
  }
  let sessionToken;
  const port = await getPort();
  const server = http.createServer().listen(port);
  const baseURL = new URL("https://ronin.co/actions/auth/cli");
  const currentHost = `http://localhost:${port}`;
  const initURL = new URL(baseURL);
  initURL.searchParams.set("from", currentHost);
  spinner.text = `Please log in using the following URL:
${initURL.href}`;
  spinner.suffixText = `
`;
  try {
    [sessionToken] = await Promise.all([
      new Promise((resolve, reject) => {
        server.once("request", (req, res) => {
          res.setHeader("connection", "close");
          const requestURL = new URL(req.url || "", currentHost);
          const requestToken = requestURL.searchParams.get("token");
          if (!requestToken) return reject(new Error("Missing token"));
          resolve(requestToken);
          const headers = { Location: baseURL.href };
          res.writeHead(307, headers).end();
        });
        server.once("error", reject);
      }),
      open(initURL.href)
    ]);
  } finally {
    server.close();
  }
  await Promise.all([
    storeSession(sessionToken),
    storeTokenForNPM(sessionToken),
    storeTokenForBun(sessionToken)
  ]);
  spinner.suffixText = "";
  spinner.succeed("Logged in successfully!");
};

// src/bin/commands/sync.ts
import ora3 from "ora";
import path4 from "path";

// src/bin/utils/schema.ts
import fs3 from "fs/promises";
import path3 from "path";
import ts from "typescript";

// src/utils/id.ts
import { init } from "@paralleldrive/cuid2";
var generateUniqueId = (length = 24) => init({ length })();

// src/bin/utils/schema.ts
async function parseSchemaDefinitionFile(filePath = "./schemas/index.d.ts") {
  const fullPath = path3.resolve(process.cwd(), filePath);
  const fileContent = await fs3.readFile(fullPath, "utf-8");
  const sourceFile = ts.createSourceFile("temp.d.ts", fileContent, ts.ScriptTarget.Latest, true);
  const results = [];
  const typeMapping = {};
  const schemaProperties = {};
  let schemaRecordAlias = "SchemaRecord";
  let schemaRecordsAlias = "SchemaRecords";
  function getFieldType(type) {
    const originalType = typeMapping[type] || type;
    switch (originalType) {
      case "ShortText":
        return "short-text";
      case "LongText":
        return "long-text";
      case "RichText":
        return "rich-text";
      case "Time":
        return "time";
      case "Blob":
        return "blob";
      case "Toggle":
        return "toggle";
      case "Number":
        return "number";
      case "Token":
        return "token";
      case "JSON":
        return "json";
      default:
        return "unknown";
    }
  }
  function parseJsDoc(node) {
    let name = "";
    let description = "";
    const details = {};
    const commentsAndTags = ts.getJSDocCommentsAndTags(node);
    if (commentsAndTags.length > 0) {
      const jsDoc = commentsAndTags[0];
      const commentText = jsDoc.comment ? ts.getTextOfJSDocComment(jsDoc.comment) : void 0;
      if (commentText) {
        const lines = commentText.split("\n").map((line) => line.trim()).filter((line) => line);
        if (lines.length > 0) {
          name = lines[0];
          description = lines.slice(1).join(" ");
        }
      }
      if ("tags" in jsDoc && jsDoc.tags) {
        jsDoc.tags.forEach((tag) => {
          const tagName = tag.tagName.getText();
          const tagComment = ts.getTextOfJSDocComment(tag.comment);
          if (tagComment) {
            details[tagName] = tagComment;
          }
        });
      }
    }
    return { name, description, details };
  }
  function visitImports(node) {
    if (ts.isImportDeclaration(node) && node.importClause && node.importClause.namedBindings) {
      const namedBindings = node.importClause.namedBindings;
      if (ts.isNamedImports(namedBindings)) {
        namedBindings.elements.forEach((element) => {
          const importedName = element.name.text;
          const originalName = element.propertyName ? element.propertyName.text : importedName;
          typeMapping[importedName] = originalName;
          if (originalName === "SchemaRecord") {
            schemaRecordAlias = importedName;
          } else if (originalName === "SchemaRecords") {
            schemaRecordsAlias = importedName;
          }
        });
      }
    }
  }
  function visitModule(node) {
    if (ts.isModuleDeclaration(node) && node.name.text === "ronin") {
      if (node.body && ts.isModuleBlock(node.body)) {
        node.body.statements.forEach((statement) => {
          if (ts.isInterfaceDeclaration(statement) && statement.name.text === "Schemas") {
            visitInterface(statement);
          }
        });
      }
    }
  }
  function visitInterface(node) {
    node.members.forEach((member) => {
      if (ts.isPropertySignature(member)) {
        const propertyName = member.name.getText();
        const typeName = member.type && ts.isTypeReferenceNode(member.type) ? member.type.typeName.getText() : "";
        if (typeName) {
          schemaProperties[propertyName] = typeName;
        }
      }
    });
    Object.keys(schemaProperties).forEach((propertyName) => {
      const typeName = schemaProperties[propertyName];
      const parsedType = parseTypeAlias(typeName, propertyName);
      if (parsedType) {
        results.push(parsedType);
      }
    });
  }
  function resolveTypeAlias(typeName) {
    let resolvedTypeName = typeName;
    let typeArguments = [];
    function typeAliasVisitor(node) {
      if (ts.isTypeAliasDeclaration(node) && node.name.text === typeName) {
        if (ts.isTypeReferenceNode(node.type) && node.type.typeName) {
          resolvedTypeName = node.type.typeName.getText();
          if (node.type.typeArguments) {
            typeArguments = Array.from(node.type.typeArguments);
          }
        }
      }
      ts.forEachChild(node, typeAliasVisitor);
    }
    ts.forEachChild(sourceFile, typeAliasVisitor);
    return { resolvedTypeName, typeArguments };
  }
  function parseTypeAlias(typeName, propertyName) {
    const result = {
      name: typeName,
      slug: propertyName,
      pluralName: "",
      pluralSlug: "",
      fields: []
    };
    Object.keys(schemaProperties).forEach((schemaPropertyName) => {
      const schemaPropertyType = schemaProperties[schemaPropertyName];
      const { resolvedTypeName, typeArguments } = resolveTypeAlias(schemaPropertyType);
      const firstTypeArgument = typeArguments[0]?.getText();
      if (resolvedTypeName === schemaRecordsAlias && typeName === firstTypeArgument) {
        result.pluralName = schemaPropertyType;
        result.pluralSlug = schemaPropertyName;
      }
    });
    function typeAliasVisitor(node) {
      if (ts.isTypeAliasDeclaration(node) && node.name.text === typeName) {
        if (ts.isTypeReferenceNode(node.type) && node.type.typeArguments) {
          const typeLiteral = node.type.typeArguments[0];
          if (ts.isTypeLiteralNode(typeLiteral)) {
            typeLiteral.members.forEach((member) => {
              if (ts.isPropertySignature(member)) {
                const fieldName = member.name.getText();
                let fieldType = getFieldType(member.type.getText());
                const { name, description, details } = parseJsDoc(member);
                if (fieldType === "unknown" && resolveTypeAlias(member.type.getText()).resolvedTypeName === schemaRecordAlias) {
                  fieldType = "record";
                }
                const field = {
                  type: fieldType,
                  id: `${fieldType}-${generateUniqueId()}`,
                  slug: fieldName,
                  name: name || fieldName.charAt(0).toUpperCase() + fieldName.slice(1),
                  description: description || void 0,
                  details,
                  unique: false
                };
                result.fields.push(field);
              }
            });
          }
        }
      }
      ts.forEachChild(node, typeAliasVisitor);
    }
    ts.forEachChild(sourceFile, typeAliasVisitor);
    if (!result.pluralName) {
      result.pluralName = typeName + "s";
      result.pluralSlug = propertyName + "s";
    }
    return result.fields.length ? result : null;
  }
  function visit(node) {
    visitImports(node);
    visitModule(node);
    ts.forEachChild(node, visit);
  }
  visit(sourceFile);
  return results;
}

// src/bin/commands/sync.ts
var sync_default = async (positionals2, token) => {
  const spinner = ora3("Reading schema definitions").start();
  const lastPositional = positionals2[positionals2.length - 1];
  const schemasDir = lastPositional === "sync" ? "schemas" : lastPositional;
  const schemaDtsFile = path4.join(process.cwd(), schemasDir, "index.d.ts");
  let status = "reading";
  try {
    const schemaJson = await parseSchemaDefinitionFile(schemaDtsFile);
    spinner.text = "Uploading schema definitions";
    status = "uploading";
    const res = await fetch("https://ronin.co/ronin/api/schemas", {
      method: "POST",
      headers: {
        "Content-Type": "application/json",
        Authorization: `Bearer ${token}`
      },
      body: JSON.stringify(schemaJson)
    });
    if (!res.ok) {
      const text = await res.text();
      throw new Error(text);
    }
  } catch (err) {
    spinner.fail(
      `Failed to ${status === "reading" ? "read schema definitions" : "apply new schema changes"}:
`
    );
    console.error(err);
    process.exit(1);
  }
  spinner.succeed("Schemas are in sync!");
};

// src/bin/utils/info.ts
import chalkTemplate from "chalk-template";

// package.json
var version = "3.2.2-leo-ron-885-experimental.28";

// src/bin/utils/info.ts
var printVersion = () => {
  console.log(version);
  process.exit(0);
};
var printHelp = () => {
  const text = chalkTemplate`
  {bold.magenta ronin} — Data at the edge

  {bold USAGE}

      {bold $} {bold.magenta ronin}
      {bold $} {bold.magenta ronin} login
      {bold $} {bold.magenta ronin} --help
      {bold $} {bold.magenta ronin} --version

  {bold COMMANDS}

      login                               Authenticate with RONIN (run by default for every command)
      init [space]                        Initialize the TypeScript types for a given space
      sync [dir]                          Read local schema definitions (from a given directory) and sync them with RONIN

  {bold OPTIONS}

      -h, --help                          Shows this help message
      -v, --version                       Shows the version of the CLI that is currently installed
      -d, --debug                         Shows additional debugging information
  `;
  console.log(text);
  process.exit(0);
};

// src/bin/index.ts
var values;
var positionals;
try {
  ({ values, positionals } = parseArgs({
    args: process.argv,
    options: {
      help: {
        type: "boolean",
        short: "h",
        default: false
      },
      version: {
        type: "boolean",
        short: "v",
        default: false
      },
      debug: {
        type: "boolean",
        short: "d",
        default: false
      }
    },
    strict: true,
    allowPositionals: true
  }));
} catch (err) {
  if (err instanceof Error) {
    console.error(err.message);
  } else {
    throw err;
  }
  process.exit(1);
}
var run = async () => {
  if (values.help) return printHelp();
  if (values.version) return printVersion();
  const normalizedPositionals = positionals.map((positional) => positional.toLowerCase());
  const appToken = process.env.RONIN_TOKEN;
  const session = await getSession();
  if (!process.stdout.isTTY && !session && !appToken) {
    let message = "If RONIN CLI is invoked from a non-interactive shell, ";
    message += "a `RONIN_TOKEN` environment variable containing an app token must be provided.";
    console.error(message);
    process.exit(1);
  }
  if (!session && !normalizedPositionals.includes("login")) await login_default(appToken);
  if (normalizedPositionals.includes("login")) return login_default(appToken);
  if (normalizedPositionals.includes("init")) return init_default(positionals);
  const token = appToken || session?.token;
  if (normalizedPositionals.includes("sync")) return sync_default(positionals, token);
  return printHelp();
};
run();
