// src/storage.ts
var extractStorableObjects = (queries) => queries.reduce((references, query, queryIndex) => {
  return [
    ...references,
    ...Object.entries(query).reduce((references2, [queryType, query2]) => {
      if (!["set", "create"].includes(queryType)) return references2;
      return [
        ...references2,
        ...Object.entries(query2).reduce((references3, [schema, instructions]) => {
          const fields = instructions[queryType === "set" ? "to" : "with"];
          return [
            ...references3,
            ...Object.entries(fields).reduce((references4, [name, value]) => {
              const isStorableObject = typeof File !== "undefined" && value instanceof File || typeof ReadableStream !== "undefined" && value instanceof ReadableStream || typeof Blob !== "undefined" && value instanceof Blob || typeof ArrayBuffer !== "undefined" && value instanceof ArrayBuffer || typeof Buffer !== "undefined" && Buffer.isBuffer(value);
              if (!isStorableObject) return references4;
              const storarableObject = {
                query: {
                  index: queryIndex,
                  type: queryType
                },
                schema,
                field: name,
                value
              };
              if ("type" in value) {
                storarableObject.contentType = value.type;
              }
              if ("name" in value) {
                storarableObject.name = value.name;
              }
              return [...references4, storarableObject];
            }, [])
          ];
        }, [])
      ];
    }, [])
  ];
}, []);
var uploadStorableObjects = async (storableObjects, options = {}) => {
  const fetcher = typeof options?.fetch === "function" ? options.fetch : fetch;
  const requests = storableObjects.map(async ({ name, value, contentType }) => {
    const headers = new Headers();
    headers.set("Authorization", `Bearer ${options.token}`);
    if (contentType) {
      headers.set("Content-Type", contentType);
    }
    if (name) {
      headers.set("Content-Disposition", `form-data; filename="${name}"`);
    }
    const request = new Request("https://storage.ronin.co/", {
      method: "PUT",
      body: value,
      headers
    });
    const response = await fetcher(request);
    if (!response.ok) throw new Error(await response.text());
    return response.json();
  });
  return Promise.all(requests).catch((err) => {
    const message = `An error occurred while uploading the binary objects included in the provided queries. ${err}`;
    throw new Error(message);
  });
};
var processStorableObjects = async (queries, upload) => {
  const objects = extractStorableObjects(queries);
  if (objects.length > 0) {
    const storedObjects = await upload(objects);
    for (let index = 0; index < objects.length; index++) {
      const { query, schema, field } = objects[index];
      const reference = storedObjects[index];
      queries[query.index][query.type][schema][query.type === "set" ? "to" : "with"][field] = reference;
    }
  }
  return queries;
};

// src/utils/helpers.ts
var objectFromAccessor = (accessor, value) => setProperty({}, accessor, value);
var getPathSegments = (path) => {
  const segments = path.replace(/\\\./g, "\u200B").split(/[[.]/g).map((s) => s.replace(/\u200B/g, ".")).filter((x) => !!x.trim()).map((x) => x.replaceAll("\\.", "."));
  return segments;
};
var setPropertyViaPathSegments = (obj, pathSegments, value) => {
  let current = obj;
  for (let i = 0; i < pathSegments.length; i++) {
    const key = pathSegments[i];
    const isLastKey = i === pathSegments.length - 1;
    if (isLastKey) {
      current[key] = typeof value === "function" ? value(current[key]) : value;
    } else {
      if (!Object.prototype.hasOwnProperty.call(current, key) || typeof current[key] !== "object") {
        current[key] = {};
      }
      current = current[key];
    }
  }
};
var setProperty = (obj, path, value) => {
  const segments = getPathSegments(path);
  setPropertyViaPathSegments(obj, segments, value);
  return obj;
};
var getPropertyViaPathSegments = (obj, pathSegments) => {
  let current = obj;
  for (const key of pathSegments) {
    if (current[key] === null || current[key] === void 0) return void 0;
    current = current[key];
  }
  return current;
};
var getProperty = (obj, path) => getPropertyViaPathSegments(obj, getPathSegments(path));
var toDashCase = (string) => {
  const capitalize = (str) => {
    const lower = str.toLowerCase();
    return lower.substring(0, 1).toUpperCase() + lower.substring(1, lower.length);
  };
  const parts = string?.replace(/([A-Z])+/g, capitalize)?.split(/(?=[A-Z])|[.\-\s_]/).map((x) => x.toLowerCase()) ?? [];
  if (parts.length === 0) return "";
  if (parts.length === 1) return parts[0];
  return parts.reduce((acc, part) => `${acc}-${part.toLowerCase()}`);
};
var formatTimeFields = (record, timeFields) => {
  timeFields.forEach(
    (field) => setProperty(record, field, (value) => value !== null ? new Date(value) : null)
  );
};

// src/utils/data-hooks.ts
var EMPTY = Symbol("empty");
var HOOK_TYPES = ["before", "during", "after"];
var getSchema = (instruction) => {
  const key = Object.keys(instruction)[0];
  let schema = String(key);
  let multipleRecords = false;
  if (schema.endsWith("s")) {
    schema = schema.substring(0, schema.length - 1);
    multipleRecords = true;
  }
  return {
    key,
    // Convert camel case (e.g. `subscriptionItems`) into slugs
    // (e.g. `subscription-items`).
    schema: toDashCase(schema),
    multipleRecords
  };
};
var getMethodName = (hookType, queryType) => {
  const capitalizedQueryType = queryType[0].toUpperCase() + queryType.slice(1);
  return hookType === "during" ? queryType : hookType + capitalizedQueryType;
};
var invokeHook = async (hookType, query, options) => {
  const { hooks, asyncContext } = options;
  const hooksForSchema = hooks[query.schema];
  const hookName = getMethodName(hookType, query.type);
  const queryInstruction = query.instruction ? structuredClone(query.instruction) : {};
  const hookArguments = [
    queryInstruction,
    query.plural,
    void 0
  ];
  if (hookType === "after") {
    const queryResult = structuredClone(query.result);
    hookArguments[2] = queryResult;
  }
  const parentHook = asyncContext.getStore();
  const shouldSkip = parentHook && (HOOK_TYPES.indexOf(hookType) <= HOOK_TYPES.indexOf(parentHook.hookType) || query.schema === parentHook.querySchema && HOOK_TYPES.indexOf(hookType) > HOOK_TYPES.indexOf(parentHook.hookType));
  if (hooksForSchema && hookName in hooksForSchema && !shouldSkip) {
    const [instructions, isMultiple, queryResults] = hookArguments;
    const hook = hooksForSchema[hookName];
    const result = await asyncContext.run(
      {
        hookType,
        queryType: query.type,
        querySchema: query.schema
      },
      async () => {
        return hookType === "after" ? await hook(instructions, isMultiple, queryResults) : await hook(instructions, isMultiple);
      }
    );
    return { ran: true, result };
  }
  return { ran: false, result: null };
};
var invokeHooks = async (hookType, modifiableQueries, modifiableResults, query, options) => {
  const queryType = Object.keys(query.definition)[0];
  const queryInstructions = query.definition[queryType];
  const { key, schema, multipleRecords } = getSchema(queryInstructions);
  const oldInstruction = queryInstructions[key];
  const executedHookResults = await invokeHook(
    hookType,
    {
      type: queryType,
      schema,
      plural: multipleRecords,
      instruction: oldInstruction,
      // For "after" hooks, we want to pass the final result associated with a
      // particular query, so that the hook can read it.
      result: hookType === "after" ? query.result : null
    },
    options
  );
  const { ran, result: hookResult } = executedHookResults;
  switch (hookType) {
    case "before":
      if (!ran) break;
      queryInstructions[key] = hookResult;
      modifiableQueries[query.index] = { [queryType]: queryInstructions };
      break;
    case "during":
      if (ran) {
        modifiableQueries[query.index] = EMPTY;
        modifiableResults[query.index] = hookResult;
      } else {
        modifiableResults[query.index] = EMPTY;
      }
      break;
    case "after":
      break;
  }
};
var runQueriesWithHooks = async (queries, options = {}) => {
  let modifiableQueries = Array.from(queries);
  const modifiableResults = new Array();
  const { hooks, waitUntil, asyncContext } = options;
  if (!hooks) return runQueries(modifiableQueries, options);
  if (typeof process === "undefined" && !waitUntil) {
    let message = 'In the case that the "ronin" package receives a value for';
    message += " its `hooks` option, it must also receive a value for its";
    message += " `waitUntil` option. This requirement only applies when using";
    message += " an edge runtime and ensures that the edge worker continues to";
    message += ' execute until all "after" hooks have been executed.';
    throw new Error(message);
  }
  if (!asyncContext) {
    let message = 'In the case that the "ronin" package receives a value for';
    message += " its `hooks` option, it must also receive a value for its";
    message += " `asyncContext` option.";
    throw new Error(message);
  }
  const hookCallerOptions = { hooks, asyncContext };
  await Promise.all(
    queries.map((query, index) => {
      return invokeHooks(
        "before",
        modifiableQueries,
        modifiableResults,
        {
          definition: query,
          index,
          result: null
        },
        hookCallerOptions
      );
    })
  );
  await Promise.all(
    queries.map((query, index) => {
      return invokeHooks(
        "during",
        modifiableQueries,
        modifiableResults,
        {
          definition: query,
          index,
          result: null
        },
        hookCallerOptions
      );
    })
  );
  modifiableQueries = modifiableQueries.filter((query) => query !== EMPTY);
  if (modifiableQueries.length === 0) return modifiableResults;
  const results = await runQueries(modifiableQueries, options);
  for (let index = 0; index < modifiableResults.length; index++) {
    const existingResult = modifiableResults[index];
    if (existingResult === EMPTY) continue;
    results.splice(index, 0, existingResult);
  }
  for (let queryIndex = 0; queryIndex < queries.length; queryIndex++) {
    const query = queries[queryIndex];
    const queryType = Object.keys(query)[0];
    if (queryType !== "set" && queryType !== "drop" && queryType !== "create") {
      continue;
    }
    const queryResult = Array.isArray(results?.[queryIndex]) ? results?.[queryIndex] : [results?.[queryIndex]];
    const promise = invokeHooks(
      "after",
      queries,
      [],
      {
        definition: query,
        index: queryIndex,
        result: queryResult
      },
      hookCallerOptions
    );
    if (waitUntil) waitUntil(promise);
  }
  return results;
};

// src/utils/errors.ts
var InvalidQueryError = class extends Error {
  message;
  query;
  path;
  details;
  constructor(details) {
    super(details.message);
    this.name = "InvalidQueryError";
    this.message = details.message;
    this.query = details.query;
    this.path = details.path;
    this.details = details.details;
  }
};
var getDotNotatedPath = (segments = []) => segments.length > 0 ? segments.reduce((path, segment, index) => {
  if (typeof segment === "number") return `${path}[${segment}]`;
  if (index === 0) return `${segment}`;
  return `${path}.${segment}`;
}, "") : null;

// src/queries.ts
var runQueries = async (queries, options = {}) => {
  const hasWriteQuery = queries.some((query) => ["create", "set", "drop"].includes(Object.keys(query)[0]));
  const hasCachingSupport = "cache" in new Request("https://ronin.co");
  const request = new Request("https://data.ronin.co/", {
    method: "POST",
    headers: {
      "Content-Type": "application/json",
      Authorization: `Bearer ${options.token}`
    },
    body: JSON.stringify({ queries }),
    // Disable cache if write queries are performed, as those must be
    // guaranteed to reach RONIN.
    ...hasWriteQuery && hasCachingSupport ? { cache: "no-store" } : {},
    // Allow for passing custom `fetch` options (e.g. in Next.js).
    ...typeof options?.fetch === "object" ? options.fetch : {}
  });
  const fetcher = typeof options?.fetch === "function" ? options.fetch : fetch;
  const response = await fetcher(request);
  const { results, error } = await response.json();
  if (error) {
    const exposedError = new Error(error.message);
    if (error.code) exposedError.code = error.code;
    throw exposedError;
  }
  const startFormatting = performance.now();
  for (let i = 0; i < results.length; i++) {
    const result = results[i];
    if ("error" in result && result.error) {
      const message = result.error.code === "BAD_REQUEST" ? "Invalid query provided." : result.error.message;
      const path = result.error.issues?.[0]?.path ? getDotNotatedPath(result.error.issues[0].path) : null;
      const query = path && typeof result.error.issues[0].path[1] === "number" ? queries[result.error.issues[0].path[1]] : null;
      const instruction = query && path ? getProperty(query, path.replace(/queries\[\d+\]\./, "")) : null;
      const details = result.error.issues?.[0] ? result.error.issues[0].message : null;
      throw new InvalidQueryError({
        message,
        query: query && path ? `${path.replace(/queries\[\d+\]\./, "")} = ${JSON.stringify(instruction)}` : null,
        path,
        details
      });
    }
    if ("amount" in result && typeof result.amount !== "undefined" && result.amount !== null) {
      results[i] = Number(result.amount);
      continue;
    }
    const timeFields = "schema" in result ? Object.entries(result.schema).filter(([, type]) => type === "time").map(([name]) => name) : [];
    if ("record" in result) {
      if (result.record === null) {
        results[i] = null;
        continue;
      }
      formatTimeFields(result.record, timeFields);
      results[i] = result.record;
      continue;
    }
    if ("records" in result) {
      for (const record of result.records) {
        formatTimeFields(record, timeFields);
      }
      if (typeof result.moreBefore !== "undefined") result.records.moreBefore = result.moreBefore;
      if (typeof result.moreAfter !== "undefined") result.records.moreAfter = result.moreAfter;
      results[i] = result.records;
      continue;
    }
  }
  const endFormatting = performance.now();
  const VERBOSE_LOGGING = typeof process !== "undefined" && process?.env && process.env.__RENDER_DEBUG_LEVEL === "verbose" || typeof import.meta?.env !== "undefined" && import.meta.env.__RENDER_DEBUG_LEVEL === "verbose";
  if (VERBOSE_LOGGING) {
    console.log(`Formatting took ${endFormatting - startFormatting}ms`);
  }
  return results;
};
var runQueriesWithStorageAndHooks = async (queries, options = {}) => {
  const queriesPopulatedWithReferences = await processStorableObjects(queries, (objects) => {
    return uploadStorableObjects(objects, options);
  });
  return runQueriesWithHooks(queriesPopulatedWithReferences, options);
};

// src/syntax/utils.ts
var IN_BATCH_ASYNC;
var IN_BATCH_SYNC = false;
var getSyntaxProxy = (queryType, queryHandler) => {
  return new Proxy(
    {},
    {
      get(_target, prop) {
        function createProxy(path) {
          const proxyTargetFunction = () => {
          };
          proxyTargetFunction();
          return new Proxy(proxyTargetFunction, {
            apply(_target2, _thisArg, args) {
              const value = args[0];
              const options = args[1];
              const expanded = objectFromAccessor(path.join("."), typeof value === "undefined" ? {} : value);
              const query = { [queryType]: expanded };
              if (IN_BATCH_ASYNC?.getStore() || IN_BATCH_SYNC) {
                return { query, options };
              }
              return queryHandler(query, options);
            },
            get(_target2, nextProp) {
              return createProxy(path.concat([nextProp]));
            }
          });
        }
        return createProxy([prop]);
      }
    }
  );
};
var getBatchProxy = (operations, options = {}, queriesHandler) => {
  let queries = [];
  if (options.asyncContext) {
    IN_BATCH_ASYNC = options.asyncContext;
    queries = IN_BATCH_ASYNC.run(true, () => operations());
  } else {
    IN_BATCH_SYNC = true;
    queries = operations();
    IN_BATCH_SYNC = false;
  }
  return queriesHandler(queries);
};

export {
  processStorableObjects,
  runQueriesWithStorageAndHooks,
  getSyntaxProxy,
  getBatchProxy
};
