import { Q as QueryHandlerOptionsFactory, R as RONIN, P as PromiseTuple } from './index-BYvMvitB.js';
import 'node:async_hooks';
import 'ronin';

/**
 * Creates a syntax factory for generating and executing queries.
 *
 * @param options - An optional object of options for the query execution.
 *
 * Alternatively, a function that returns the object may be provided instead,
 * which is useful for cases in which the config must be generated dynamically
 * whenever a query is executed.
 *
 * @returns An object with methods for generating and executing different types
 * of queries.
 *
 * ### Usage
 * ```typescript
 * const { get, set, create, count, drop } = createSyntaxFactory({
 *   token: '...'
 * });
 *
 * await get.accounts();
 *
 * await set.account({
 *   with: {
 *     email: 'mike@gmail.com',
 *   },
 *   to: {
 *     status: 'active',
 *   },
 * });
 *
 *
 * await create.account({ with: { email: 'mike@gmail.com' } });
 *
 * await count.accounts();
 *
 * await drop.accounts.with.emailVerified.notBeing(true);
 *
 * // Execute a batch of operations
 * const batchResult = await batch(() => [
 *   get.accounts(),
 *   get.account.with.email('mike@gmail.com')
 * ]);
 * ```
 */
declare const createSyntaxFactory: (options: QueryHandlerOptionsFactory) => {
    create: RONIN.Creator;
    get: RONIN.Getter;
    set: RONIN.Setter;
    drop: RONIN.Dropper;
    count: RONIN.Counter;
    batch: <T extends [Promise<any>, ...Promise<any>[]] | Promise<any>[]>(operations: () => T, batchQueryOptions?: Record<string, unknown>) => Promise<PromiseTuple<T>>;
};

declare const create: RONIN.Creator<undefined>;
declare const get: RONIN.Getter<undefined>;
declare const set: RONIN.Setter<undefined>;
declare const drop: RONIN.Dropper<undefined>;
declare const count: RONIN.Counter<undefined>;
declare const batch: <T extends [Promise<any>, ...Promise<any>[]] | Promise<any>[]>(operations: () => T, batchQueryOptions?: Record<string, unknown>) => Promise<PromiseTuple<T>>;

export { RONIN, batch, count, create, createSyntaxFactory as default, drop, get, set };
