import { a as Query, b as QueryHandlerOptions, c as Results, S as StorableObject, d as StoredObject, e as QueryItem, P as PromiseTuple } from '../index-BYvMvitB.js';
import 'node:async_hooks';
import 'ronin';

/**
 * Runs a list of `Query`s.
 *
 * @param queries - A list of queries to execute.
 * @param options - A list of options to change how the queries are executed.
 *
 * @returns The results of the queries that were passed.
 */
declare const runQueriesWithStorageAndHooks: <T>(queries: Query[], options?: QueryHandlerOptions) => Promise<Results<T>>;

/**
 * ## 🚧 For internal use only! 🚧
 *
 * Process `StorableObjectValue`s contained in queries.
 *
 * @param queries - Array of queries that contain `StorableObjectValue`s.
 * @param upload - A function that receives `StorableObject`s and uploads them.
 *
 * @returns Queries with `StorableObjectValue`s replaced by `StoredObject`s.
 */
declare const processStorableObjects: (queries: Query[], upload: (objects: StorableObject[]) => Promise<StoredObject[]> | StoredObject[]) => Promise<Query[]>;

/**
 * A utility function that creates a Proxy object to handle dynamic property
 * access and function calls. It is used to create a syntax that allows for
 * dynamic query generation.
 *
 * @param queryType - The type of the query. This will be used as the key in
 * the generated query object.
 * @param queryHandler - A function that handles the execution of the query.
 *
 * @returns A Proxy object that intercepts property access and function
 * calls to generate and execute queries.
 *
 * ### Usage
 * ```typescript
 * const proxy = getSyntaxProxy('get', async (query) => {
 *   // Execute the query and return the result
 * });
 *
 * const result = await get.account();
 *
 * const result = await get.account.with.email('mike@gmail.com');
 * ```
 */
declare const getSyntaxProxy: (queryType: string, queryHandler: (query: Query, options?: Record<string, unknown>) => Promise<any> | any) => any;
/**
 * Executes a batch of operations and handles their results. It is used to
 * execute multiple queries at once and return their results at once.
 *
 * @param operations - A function that returns an array of Promises. Each
 * Promise should resolve with a Query object.
 * @param queriesHandler - A function that handles the execution of the queries.
 * This function is expected to receive an array of Query objects and return a
 * Promise that resolves with the results of the queries.
 *
 * @returns A Promise that resolves with a tuple of the results of the queries.
 *
 * ### Usage
 * ```typescript
 * const results = await batch(() => [
 *   get.accounts(),
 *   get.account.with.email('mike@gmail.com')
 * ], async (queries) => {
 *   // Execute the queries and return their results
 * });
 * ```
 */
declare const getBatchProxy: <T extends [Promise<any> | any, ...(Promise<any> | any)[]] | (Promise<any> | any)[]>(operations: () => T, options: QueryHandlerOptions | undefined, queriesHandler: (queries: QueryItem[], options?: Record<string, unknown>) => Promise<any> | any) => Promise<PromiseTuple<T>> | T;

export { getBatchProxy, getSyntaxProxy, processStorableObjects, runQueriesWithStorageAndHooks as runQueries };
